import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.awt.Color;
import java.util.HashMap;

/**
 * A piano that can be played with the computer keyboard.
 *
 * @author: M. Kolling, Ed Parrish
 * @version: 1.0  11/15/2010
 */
public class Piano extends World
{
    private static final int SCENARIO_WIDTH = 800;
    private static final int SCENARIO_HEIGHT = 340;

    private String[] whiteKeyNames =
        {"A", "S", "D", "F", "G", "H", "J", "K", "L", ";", "'", "\\"};
    private  String[] whiteNotes =
        {"3c", "3d", "3e", "3f", "3g", "3a", "3b", "4c", "4d", "4e", "4f", "4g"};

    private String[] blackKeyNames =
        {"W", "E", "", "T", "Y", "U", "", "O", "P", "", "]"};
    private String[] blackNotes =
        {"3c#", "3d#", "", "3f#", "3g#", "3a#", "", "4c#", "4d#", "", "4f#"};

    private SongPlayer player;
    private HashMap<String, Key> keys = new HashMap<String, Key>();
    private boolean autoplay;

    /**
     * Construct the piano.
     */
    public Piano()
    {
        super(SCENARIO_WIDTH, SCENARIO_HEIGHT, 1);
        makeKeys();
        player = new SongPlayer();
        addObject(player, 30, 310);
        makeSongs();
    }

    /**
     * Make the piano keys.
     */
    public void makeKeys() {
        final int OFFSET = 20;
        final int WHT_START = 54;
        final int BLK_START = 85;

        GreenfootImage whiteUp = new GreenfootImage("white-key.png");
        GreenfootImage blackUp = new GreenfootImage("black-key.png");
        GreenfootImage whiteDown = new GreenfootImage("white-key-down.png");
        GreenfootImage blackDown = new GreenfootImage("black-key-down.png");

        int width = whiteUp.getWidth();
        int height = whiteUp.getHeight();
        int heightBlk = blackUp.getHeight();

        Key key = null;
        for (int i = 0; i < whiteKeyNames.length; i++) {
            key = new Key(whiteKeyNames[i], whiteNotes[i] + ".wav",
                whiteUp, whiteDown);
            keys.put(whiteKeyNames[i], key);
            keys.put(whiteNotes[i], key);
            addObject(key, WHT_START + i * width, height / 2);
            Label label = new Label(whiteKeyNames[i]);
            addObject(label, WHT_START + i * width, height - OFFSET);
        }

        for (int i = 0; i < blackKeyNames.length; i++) {
            if (!blackKeyNames[i].equals("")) {
                key = new Key(blackKeyNames[i], blackNotes[i] + ".wav",
                    blackUp, blackDown);
                keys.put(blackKeyNames[i], key);
                keys.put(blackNotes[i], key);
                addObject(key, BLK_START + i * width, heightBlk / 2);
                Label label = new Label(blackKeyNames[i], Color.WHITE);
                addObject(label, BLK_START + i * width, heightBlk - OFFSET);
            }
        }

        String instr = "Click 'Run', then use the computer keyboard to play.";
        instr += " Press the number keys to play prerecorded songs.";
        Label instructions = new Label(instr, Color.WHITE);
        int labelWidth = instructions.getImage().getWidth();
        addObject(instructions, getWidth() / 2, 310);
    }

    /**
     * Set the automatic player mode.
     *
     * @param auto Set true for automatic player mode, false for manual mode.
     */
    public void setAutoPlay(boolean auto)
    {
        autoplay = auto;
    }

    /**
     * Test if auto-play mode is set or not.
     *
     * @return true if auto-play mode; otherwise false.
     */
    public boolean getAutoPlay()
    {
        return autoplay;
    }

    /**
     * Makes sample songs and adds them to the song player.
     */
    public void makeSongs()
    {
        // Fur Elise -- using note names
        Song song = new Song();
        song.addNote(new Note(keys.get("4e"), 10, 10));
        song.addNote(new Note(keys.get("4d#"), 20, 10));
        song.addNote(new Note(keys.get("4e"), 30, 10));
        song.addNote(new Note(keys.get("4d#"), 40, 10));
        song.addNote(new Note(keys.get("4e"), 50, 10));
        song.addNote(new Note(keys.get("3b"), 60, 10));
        song.addNote(new Note(keys.get("4d"), 70, 10));
        song.addNote(new Note(keys.get("4c"), 80, 10));
        song.addNote(new Note(keys.get("3a"), 90, 20));
        song.addNote(new Note(keys.get("3c"), 120, 10));
        song.addNote(new Note(keys.get("3e"), 130, 10));
        song.addNote(new Note(keys.get("3a"), 140, 10));
        song.addNote(new Note(keys.get("3b"), 150, 20));
        song.addNote(new Note(keys.get("3e"), 180, 10));
        song.addNote(new Note(keys.get("3g#"), 190, 10));
        song.addNote(new Note(keys.get("3b"), 200, 10));
        song.addNote(new Note(keys.get("4c"), 210, 10));
        player.addSong(song);

        // Scarborough Fair -- using computer keyboard names
        Song sf = new Song();
        sf.addNote(new Note(keys.get("S"), 10, 19));
        sf.addNote(new Note(keys.get("S"), 30, 10));
        sf.addNote(new Note(keys.get("H"), 40, 19));
        sf.addNote(new Note(keys.get("H"), 60, 10));
        sf.addNote(new Note(keys.get("D"), 70, 15));
        sf.addNote(new Note(keys.get("F"), 85, 5));
        sf.addNote(new Note(keys.get("D"), 90, 10));
        sf.addNote(new Note(keys.get("S"), 100, 30));
        sf.addNote(new Note(keys.get("H"), 140, 10));
        sf.addNote(new Note(keys.get("K"), 150, 10));
        sf.addNote(new Note(keys.get("L"), 160, 20));
        sf.addNote(new Note(keys.get("K"), 180, 10));
        sf.addNote(new Note(keys.get("H"), 190, 10));
        sf.addNote(new Note(keys.get("J"), 200, 10));
        sf.addNote(new Note(keys.get("G"), 210, 10));
        sf.addNote(new Note(keys.get("H"), 220, 30));
        player.addSong(sf);

        // Jingle Bells -- with harmony
        Song jingle = new Song();
        jingle.addNote(new Note(keys.get("3b"), 10, 9));
        jingle.addNote(new Note(keys.get("3g"), 10, 9));
        jingle.addNote(new Note(keys.get("3b"), 20, 9));
        jingle.addNote(new Note(keys.get("3g"), 20, 9));
        jingle.addNote(new Note(keys.get("3b"), 30, 19));
        jingle.addNote(new Note(keys.get("3g"), 30, 19));
        jingle.addNote(new Note(keys.get("3b"), 50, 9));
        jingle.addNote(new Note(keys.get("3g"), 50, 9));
        jingle.addNote(new Note(keys.get("3b"), 60, 9));
        jingle.addNote(new Note(keys.get("3g"), 60, 9));
        jingle.addNote(new Note(keys.get("3b"), 70, 19));
        jingle.addNote(new Note(keys.get("3g"), 70, 19));
        jingle.addNote(new Note(keys.get("3b"), 90, 9));
        jingle.addNote(new Note(keys.get("3g"), 90, 9));
        jingle.addNote(new Note(keys.get("4d"), 100, 9));
        jingle.addNote(new Note(keys.get("3g"), 100, 9));
        jingle.addNote(new Note(keys.get("3g"), 110, 14));
        jingle.addNote(new Note(keys.get("3d"), 110, 14));
        jingle.addNote(new Note(keys.get("3a"), 125, 4));
        jingle.addNote(new Note(keys.get("3d"), 125, 4));
        jingle.addNote(new Note(keys.get("3b"), 130, 20));
        jingle.addNote(new Note(keys.get("3g"), 130, 20));
        player.addSong(jingle);
    }
}
